'use strict';

let Util = module.exports = {};

const _ = require('lodash');
const register = require('./register');
const cortex = require('@dp/cortex4n');

/**
 * convert widget config list 2 cortex execute object for static resource to include
 * @param {KOA} koa [middleware context]
 * @param  {object} widgets     [widgets array]
 * @param  {object} options [options from action]
 * @return {object} result [cortex execute results]
 */
Util.loadCortexResults = function *(koa, widgets, options) {
    //  get all facades,both widgets & main path
    let facades = collectAllFacades(widgets, options);

    //  traversal all facades,collect data & styles for every entry
    let facadeObj = convertFacades2EntryMap(facades);

    //  add to array,make styles array to styles string
    let keys = _.keys(facadeObj);
    let results = [];
    _.each(keys, (key) => {
        let val = facadeObj[key];
        results.push({
            entry: val.entry,
            data: {},
            styles: _.uniq(val.styles).join(',') // unique
        });
    });

    return yield cortex.parse('facades', {
        facades: results,
        cortexPath: koa.cookies.get('cortex_path')
    });
};

/**
 * collect all facades
 * @param  {array} widgets [widget config list]
 * @param  {object} options [option from action]
 * @return {[type]}         [description]
 */
function collectAllFacades(widgets, options) {
    let retarr = [];
    //  add option facades
    retarr = retarr.concat(options.facades || []);
    //  add widgets
    _.each(widgets, (item) => {
        let widget = register.getByName(item.name);
        retarr.push({
            entry: widget.entry,
            data: widget.data,
            styles: widget.styles
        });
    });
    return retarr;
}

/**
 * convert facade array to map which key is entry
 * @param  {array} facades [all facades]
 * @return {object}         [facades map]
 */
function convertFacades2EntryMap(facades) {
    let retObj = {};
    _.each(facades, (facade) => {
        let entry = facade.entry;
        if (!entry) {
            return true;
        }
        let oriResult = retObj[entry] || {};
        oriResult.entry = entry;
        //  extends data
        oriResult.data = _.extend({}, oriResult.data, facade.data);
        // styles as arr,because maybe one entry to many styleses
        oriResult.styles = oriResult.styles || [];
        oriResult.styles.push(facade.styles);

        retObj[entry] = oriResult;
    });
    return retObj;
}
