'use strict';

const Chicken = module.exports = require('./chicken');

const path = require('path');
const fs = require('fs');
const _ = require('lodash');
const executor = require('./executor');
const render = require('./render');
const register = require('./register');
const apirouter = require('./apirouter');
const routerbinder = require('./routerbinder');

const renderOptions = {};
const registerOptions = {};
const actionOptions = {};

const contextFile = 'context.json';

Chicken.use = function(options) {
    let koa = options.app;
    let appRoot = options.appRoot;
    Chicken.context.appRoot = appRoot;
    loadContext(appRoot);
    render.init(renderOptions);
    register.init(registerOptions);
    koa.use(apirouter);
    koa.use(routerbinder.init(actionOptions));
}


function loadContext(appRoot) {
    let cfgFile = path.resolve(appRoot, contextFile);
    var configuration = JSON.parse(fs.readFileSync(cfgFile));
    _.extend(renderOptions, configuration.render);
    _.extend(registerOptions, configuration.widgets);
    _.extend(actionOptions, configuration.actions);
}

Chicken.execute = executor.execute;
Chicken.render = render.render;