'use strict';
let Register = module.exports = {};

const _ = require('lodash');
const os = require('os');
const fs = require('fs');
const context = require('./chicken').context;
const path = require('path');


const DEFAULT_OPTION = {
    widgetDir: './widgets',
    logPath: 'widgets.json'
};

let name2Module = {};

/**
 * init widget, it will scan the config folder and register each widget to the container
 * @param {object} options [config object]
 *
 * example:
 * {
 *      widgetDir: [fold where widget lives],
 *      widgetListPath: [file which lists all the widgets of the application,just for find]
 * }
 * @return {void}
 */
Register.init = function(options) {
    options = _.extend({}, DEFAULT_OPTION, options);
    let abWidgetDir = path.resolve(context.appRoot, options.widgetDir);
    let files = getAllFilesSync(abWidgetDir) || [];

    _.each(files, (file) => {
        console.log('>>>>>%s', file);
        let widget = require(file);
        if (!isWidget(widget)) {
            return true;__dirname
        }
        widget.filePath = file;

        let name = widget.name;
        if (name2Module[name]) {
            throw new Error('Duplicate define widget name: ' + name + ',please check widgets');
        }
        name2Module[name] = widget;
    });

    writeWidgetListSync(options);
}


/**
 * 根据name获取Pagelet
 * @param name
 * @returns {*}
 */
Register.getByName = function(name) {
    return name2Module[name];
}


function writeWidgetListSync(options) {
    if (!name2Module) {
        return;
    }
    if (!(options && options.widgetDir && options.logPath)) {
        return;
    }

    let widgetDir = path.resolve(context.appRoot, options.widgetDir);;
    let logPath = path.resolve(context.appRoot, options.logPath);

    if (!(_.isString(widgetDir) && _.isString(logPath))) {
        return;
    }

    let names = _.keys(name2Module);
    let contents = ['widget：'.concat(os.EOL)];
    _.each(names, (name) => {
        let config = name2Module[name];
        let contentStr = '';
        let describes = config.describe.split('\n');

        contentStr = contentStr.concat('======='.repeat(15));
        contentStr = contentStr.concat(os.EOL).concat(os.EOL);
        contentStr = contentStr.concat('组件名称：').concat(os.EOL).concat('\t' + name).concat(os.EOL);
        contentStr = contentStr.concat('文件路径：').concat(os.EOL).concat('\t' + config.filePath.replace(widgetDir, '')).concat(os.EOL);
        contentStr = contentStr.concat('描述：').concat(os.EOL).concat('\t' + describes.join(os.EOL));
        contentStr = contentStr.concat(os.EOL);
        contents.push(contentStr);
    });
    fs.writeFileSync(logPath, contents.join(os.EOL), 'utf-8');
}


function isWidget(widget) {
    return (widget && widget.name && _.isFunction(widget.execute));
}

function getAllFilesSync(folder) {
    let rets = [];
    let files = fs.readdirSync(folder);

    files.forEach((file) => {
        let pathname = folder + '/' + file;
        let stat = fs.lstatSync(pathname);

        if (!stat.isDirectory()) {
            rets.push(pathname);
        } else {
            rets = rets.concat(getAllFilesSync(pathname));
        }
    });
    return rets;
}
