'use strict';

const Render = module.exports = {};

const context = require('./chicken').context;
const path = require('path');
const fs = require('co-fs');
const _ = require('lodash');


const default_setting = {
    cache: true,
    layout: 'layout',
    viewExt: 'html'
};

let globalSettings = {};
let globalCache = {};


/**
 * 初始化接口
 * @param settings
 */
Render.init = function(settings) {
    if (!settings || !settings.view) {
        throw new Error('settings.root required');
    }

    _.extend(globalSettings, default_setting, settings);
    globalSettings.viewExt = globalSettings.viewExt ? '.' + globalSettings.viewExt.replace(/^\./, '') : '';

    Render.init = function() {
        throw new Error('can not be inited twice');
    };
};

/**
 * 外部渲染接口
 * @param view
 * @param model
 * @param options
 * @returns {*}
 */
Render.render = function *render(view, model, options) {
    let _model = _.extend({}, model, options);

    let html = yield * compile(view, _model);

    let layout = _model.layout === false ? false : (_model.layout || globalSettings.layout);
    if (layout) {
        _model.body = html;
        html = yield * render(layout, _model);
    }

    return html;
};


function *compile(view, model) {
    let settings    =   globalSettings,
        _view       =   view,
        viewExt     =   settings.viewExt || '',
        viewAbPath  =   path.resolve(context.appRoot, settings.view),
        viewPath    =   path.join(viewAbPath, _view + viewExt);

    let compiledFn;

    if (settings.cache && globalCache[viewPath]) {
        compiledFn = globalCache[viewPath];
    } else {
        let tpl = yield fs.readFile(viewPath, 'utf8');
        compiledFn = _.template(tpl);
        if (settings.cache) {
            globalCache[viewPath] = compiledFn;
        }
    }
    return compiledFn.call(model.scope, model);
}
