'use strict';

require('rootpath')();

const os = require('os');
const fs = require('fs');
const path = require('path');
const _ = require('lodash');
const router = require('koa-router')();
const util = require('./util');
const context = require('./chicken').context;

const default_setting = {
    actionDir: 'action',
    apiLogPath: ''
};

const default_methods = ['get', 'post'];

/**
 * 日志api清单,方便查阅
 */
function writeToLog(api2path, relativePath) {
    if (!(api2path && relativePath)) {
        return;
    }
    let apis = _.keys(api2path),
        contents = ['api清单：', os.EOL, '======='.repeat(15)];
    _.each(apis, (api) => {
        let filePath = api2path[api] || '',
            contentStr = '';
        contentStr = contentStr.concat('api路径：').concat(os.EOL).concat('\t' + api).concat(os.EOL);
        contentStr = contentStr.concat('文件路径：').concat(os.EOL).concat('\t' + filePath.replace(__dirname, '')).concat(os.EOL);
        contentStr = contentStr.concat(os.EOL.repeat(3));
        contentStr = contentStr.concat(os.EOL.repeat(3));
        contentStr = contentStr.concat('======='.repeat(15));
        contents.push(contentStr);
    });

    fs.writeFileSync(relativePath, contents.join(os.EOL.repeat(3)), 'utf-8');
}

function isAction(fileConfig) {
    return fileConfig && fileConfig.url && _.isFunction(fileConfig.action);
}

function init(options) {
    options             =   _.extend({}, default_setting, options);
    let abActionPath    =   path.resolve(context.appRoot, options.actionDir),
        actionDir       =   path.resolve(__dirname, abActionPath),
        files           =   util.getAllFiles(actionDir) || [],
        fileConfig      =   undefined,
        methods         =   undefined,
        action          =   undefined,
        api2path        =   {};


    _.each(files, function(file) {
        fileConfig = require(file);
        if (!isAction(fileConfig)) {
            return true;
        }

        methods = fileConfig.methods || default_methods;
        methods = Array.isArray(methods) ? methods : [methods];

        action = fileConfig.action;
        router.register(fileConfig.url, methods, [action]);
        if (api2path[fileConfig.url]){
            throw new Error('Duplicate define router url: ' + fileConfig.url + ',please check actions');
        }
        api2path[fileConfig.url] = file;
    });

    if (!_.isEmpty(options.apiLogPath)) {
        writeToLog(api2path, options.apiLogPath);
    }
    return router.routes();
}

module.exports = {
    init
};
